"""
$Id: base.py 1808 2007-02-06 11:39:11Z hazmat $
"""

from zope.interface import implements
from zope.component import getUtility

from zope.viewlet.interfaces import IViewlet

from DateTime import DateTime

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import ITranslationServiceTool

from plone.app.iterate.util import get_storage
from plone.app.iterate.interfaces import keys

from plone.app.iterate.relation import WorkingCopyRelation

from plone.memoize.instance import memoize

class BaseInfoViewlet( BrowserView ):
    
    implements( IViewlet )
    
    def __init__( self, context, request, view, manager ):
        super( BaseInfoViewlet, self ).__init__( context, request )
        self.__parent__ = view
        self.view = view
        self.manager = manager
        
    def update( self ):
        pass
    
    def render(self):
        raise NotImplementedError(
            '`render` method must be implemented by subclass.')
    
    @memoize
    def created( self ):
        time = self.properties.get( keys.checkout_time, DateTime() )
        util = getUtility(ITranslationServiceTool)
        return util.ulocalized_time(time, None, self.context, domain='plonelocales')

    @memoize
    def creator( self ):
        user_id = self.properties.get( keys.checkout_user )
        membership = getToolByName( self.context, 'portal_membership' )
        if not user_id:
            return membership.getAuthenticatedMember()
        return membership.getMemberById( user_id )
        
    @memoize
    def creator_url( self ):
        creator = self.creator()
        portal_url = getToolByName( self.context, 'portal_url' )
        return "%s/author/%s" % ( portal_url(), creator.getId() )
        
    @memoize
    def creator_name( self ):
        creator = self.creator()
        return creator.getProperty('fullname') or creator.getUserName()

    @property
    @memoize
    def properties( self ):
        wc_ref = self._getReference()
        return get_storage( wc_ref )

    def _getReference( self ):
        raise NotImplemented
        
class BaselineInfoViewlet( BaseInfoViewlet ):
    
    render = ViewPageTemplateFile('info_baseline.pt')

    def working_copy( self ):
        return self.context.getBRefs( WorkingCopyRelation.relationship )[0]

    def _getReference( self ):
        refs = self.context.getBackReferenceImpl( WorkingCopyRelation.relationship )
        wc_ref = refs[0]
        return wc_ref
        
class CheckoutInfoViewlet( BaseInfoViewlet ):
    
    render = ViewPageTemplateFile('info_checkout.pt')
    
    def baseline( self ):
        return self.context.getReferences( WorkingCopyRelation.relationship )[0]
    
    def _getReference( self ):
        refs = self.context.getReferenceImpl( WorkingCopyRelation.relationship )
        wc_ref = refs[0]
        return wc_ref
        



##################################################################
#
# (C) Copyright 2006 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with iterate; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################

from zope.interface import implements
from zope.component import adapts

from Acquisition import aq_inner, aq_parent
from AccessControl import getSecurityManager
from Products.CMFCore.interfaces import IDynamicType
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import AddPortalContent

from plone.app.iterate.interfaces import IWCContainerLocator

class HomeFolderLocator(object):
    """Locate the current user's home folder, if possible.
    """
    implements(IWCContainerLocator)
    adapts(IDynamicType)
    
    def __init__(self, context):
        self.context = context
    
    title = u"Home folder"
    
    @property
    def available(self):
        return self() is not None
        
    def __call__(self):
        return getToolByName(aq_inner(self.context), 'portal_membership').getHomeFolder()
        
class ParentFolderLocator(object):
    """Locate the parent of the context, if the user has the
    Add portal content permission.
    """
    implements(IWCContainerLocator)
    adapts(IDynamicType)
    
    def __init__(self, context):
        self.context = context
    
    title = u"Parent folder"
    
    @property
    def available(self):
        return bool(getSecurityManager().checkPermission(AddPortalContent, aq_inner(self.context)))
        
    def __call__(self):
        if not self.available:
            return None
        return aq_parent(aq_inner(self.context))


##################################################################
#
# (C) Copyright 2006-2007 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
$Id: archiver.py 1824 2007-02-08 17:59:41Z hazmat $
"""

from Products.CMFCore.utils import getToolByName

from zope.interface import implements
from zope.component import adapts

import interfaces

class ContentArchiver( object ):
    
    implements( interfaces.IObjectArchiver )
    adapts( interfaces.IIterateAware )

    def __init__( self, context ):
        self.context = context
    
    def save( self, checkin_message ):
        repository = getToolByName( self.context, 'portal_repository')
        repository.save( self.context, checkin_message )

    def isVersionable( self ):
        repository = getToolByName( self.context, 'portal_repository')
        if not repository.isVersionable( self.context ):
            return False
        return True

    def isVersioned( self ):
        archivist = getToolByName( self.context, 'portal_archivist')
        version_count = len( archivist.queryHistory( self.context ) )
        return bool( version_count )

    def isModified( self ):
        repository = getToolByName( self.context, 'portal_repository')

        try:
            return not repository.isUpToDate( self.context )
        except:
            return False
        
        
    
